@@ -72,32 +72,6 @@ _Pycairo_Get_Error(void) {
 #endif
 }
 
-
-int
-Pycairo_Check_Status (cairo_status_t status) {
-  if (PyErr_Occurred() != NULL)
-    return 1;
-
-  switch (status) {
-  case CAIRO_STATUS_SUCCESS:
-    return 0;
-    /* if appropriate - translate the status string into Python,
-     * else - use cairo_status_to_string()
-     */
-  case CAIRO_STATUS_NO_MEMORY:
-    PyErr_NoMemory();
-    break;
-  case CAIRO_STATUS_READ_ERROR:
-  case CAIRO_STATUS_WRITE_ERROR:
-    PyErr_SetString(PyExc_IOError, cairo_status_to_string (status));
-    break;
-  default:
-    Pycairo_Set_Error(status);
-  }
-  return 1;
-}
-
-
 /* C API.  Clients get at this via Pycairo_IMPORT or import_cairo(), defined in pycairo.h.
  */
 static Pycairo_CAPI_t CAPI = {
@@ -260,10 +234,40 @@ PYCAIRO_MOD_INIT(_cairo)
     return PYCAIRO_MOD_ERROR_VAL;
   if (PyType_Ready(&PycairoRadialGradient_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoMeshPattern_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoRasterSourcePattern_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
 
   if (PyType_Ready(&PycairoRectangleInt_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 
+  if (PyType_Ready(&PycairoDevice_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+  PycairoGlyph_Type.tp_base = &PyTuple_Type;
+  if (PyType_Ready(&PycairoGlyph_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+  PycairoRectangle_Type.tp_base = &PyTuple_Type;
+  if (PyType_Ready(&PycairoRectangle_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+  PycairoTextCluster_Type.tp_base = &PyTuple_Type;
+  if (PyType_Ready(&PycairoTextCluster_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+  PycairoTextExtents_Type.tp_base = &PyTuple_Type;
+  if (PyType_Ready(&PycairoTextExtents_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  if (PyType_Ready(&PycairoScriptDevice_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoScriptSurface_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+#endif
+
   if (PyType_Ready(&PycairoRegion_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 
@@ -275,6 +279,8 @@ PYCAIRO_MOD_INIT(_cairo)
 #ifdef CAIRO_HAS_IMAGE_SURFACE
   if (PyType_Ready(&PycairoImageSurface_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoMappedImageSurface_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
 #endif
 #ifdef CAIRO_HAS_PDF_SURFACE
   if (PyType_Ready(&PycairoPDFSurface_Type) < 0)
@@ -306,6 +312,10 @@ PYCAIRO_MOD_INIT(_cairo)
   if (PyType_Ready(&PycairoXlibSurface_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 #endif
+#ifdef CAIRO_HAS_TEE_SURFACE
+  if (PyType_Ready(&PycairoTeeSurface_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+#endif
 
 #if PY_MAJOR_VERSION < 3
   m = Py_InitModule("cairo._cairo", cairo_functions);
@@ -359,6 +369,12 @@ PYCAIRO_MOD_INIT(_cairo)
   Py_INCREF(&PycairoRadialGradient_Type);
   PyModule_AddObject(m, "RadialGradient",
 		     (PyObject *)&PycairoRadialGradient_Type);
+  Py_INCREF(&PycairoRadialGradient_Type);
+  PyModule_AddObject(m, "MeshPattern",
+                     (PyObject *)&PycairoMeshPattern_Type);
+  Py_INCREF(&PycairoRasterSourcePattern_Type);
+  PyModule_AddObject(m, "RasterSourcePattern",
+                     (PyObject *)&PycairoRasterSourcePattern_Type);
 
   Py_INCREF(&PycairoRectangleInt_Type);
   PyModule_AddObject(m, "RectangleInt",  (PyObject *)&PycairoRectangleInt_Type);
@@ -372,6 +388,28 @@ PYCAIRO_MOD_INIT(_cairo)
   Py_INCREF(&PycairoSurface_Type);
   PyModule_AddObject(m, "Surface", (PyObject *)&PycairoSurface_Type);
 
+  Py_INCREF(&PycairoDevice_Type);
+  PyModule_AddObject(m, "Device", (PyObject *)&PycairoDevice_Type);
+
+  Py_INCREF(&PycairoGlyph_Type);
+  PyModule_AddObject(m, "Glyph", (PyObject *)&PycairoGlyph_Type);
+
+  Py_INCREF(&PycairoRectangle_Type);
+  PyModule_AddObject(m, "Rectangle", (PyObject *)&PycairoRectangle_Type);
+
+  Py_INCREF(&PycairoTextCluster_Type);
+  PyModule_AddObject(m, "TextCluster", (PyObject *)&PycairoTextCluster_Type);
+
+  Py_INCREF(&PycairoTextExtents_Type);
+  PyModule_AddObject(m, "TextExtents", (PyObject *)&PycairoTextExtents_Type);
+
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  Py_INCREF(&PycairoScriptDevice_Type);
+  PyModule_AddObject(m, "ScriptDevice", (PyObject *)&PycairoScriptDevice_Type);
+  Py_INCREF(&PycairoScriptSurface_Type);
+  PyModule_AddObject(m, "ScriptSurface", (PyObject *)&PycairoScriptSurface_Type);
+#endif
+
 #ifdef CAIRO_HAS_IMAGE_SURFACE
   Py_INCREF(&PycairoImageSurface_Type);
   PyModule_AddObject(m, "ImageSurface",
@@ -420,21 +458,21 @@ PYCAIRO_MOD_INIT(_cairo)
 		     (PyObject *)&PycairoXlibSurface_Type);
 #endif
 
+#ifdef CAIRO_HAS_TEE_SURFACE
+  Py_INCREF(&PycairoTeeSurface_Type);
+  PyModule_AddObject(m, "TeeSurface",
+		     (PyObject *)&PycairoTeeSurface_Type);
+#endif
+
   /* Add 'cairo.Error' to the module */
-#if PY_MAJOR_VERSION >= 3
-  GETSTATE(m)->ErrorObject = error_get_type();
-  if (GETSTATE(m)->ErrorObject == NULL) {
-    Py_DECREF(m);
+  error = error_get_type();
+  if (error == NULL)
     return PYCAIRO_MOD_ERROR_VAL;
-  }
-  error = GETSTATE(m)->ErrorObject;
+
+#if PY_MAJOR_VERSION >= 3
+  GETSTATE(m)->ErrorObject = error;
 #else
-  if (_CairoError == NULL) {
-    _CairoError = error_get_type();
-    if (_CairoError == NULL)
-      return PYCAIRO_MOD_ERROR_VAL;
-  }
-  error = _CairoError;
+  _CairoError = error;
 #endif
 
   Py_INCREF(error);
@@ -527,6 +565,11 @@ PYCAIRO_MOD_INIT(_cairo)
 #else
   PyModule_AddIntConstant(m, "HAS_MIME_SURFACE", 0);
 #endif
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  PyModule_AddIntConstant(m, "HAS_SCRIPT_SURFACE", 1);
+#else
+  PyModule_AddIntConstant(m, "HAS_SCRIPT_SURFACE", 0);
+#endif
 
 #define STRCONSTANT(x) PyModule_AddStringConstant(m, #x, CAIRO_##x)
 
