@@ -43,8 +43,15 @@
 #include "compat.h"
 
 
+cairo_glyph_t * _PycairoGlyphs_AsGlyphs (PyObject *py_object, int *num_glyphs);
+int _PyGlyph_AsGlyph (PyObject *pyobj, cairo_glyph_t *glyph);
+int _PyTextCluster_AsTextCluster (PyObject *pyobj,
+                                  cairo_text_cluster_t *cluster);
+
 PyObject *_Pycairo_Get_Error(void);
-void Pycairo_Set_Error (cairo_status_t status);
+
+PyObject* Pycairo_richcompare (void* a, void *b, int op);
+PyObject* Pycairo_tuple_getattro (PyObject *self, char **kwds, PyObject *name);
 
 extern PyTypeObject PycairoContext_Type;
 PyObject *PycairoContext_FromContext (cairo_t *ctx, PyTypeObject *type,
@@ -75,6 +82,12 @@ extern PyTypeObject PycairoRadialGradient_Type;
 PyObject *PycairoPattern_FromPattern (cairo_pattern_t *pattern,
 				      PyObject *base);
 
+typedef PycairoPattern PycairoMeshPattern;
+extern PyTypeObject PycairoMeshPattern_Type;
+
+typedef PycairoPattern PycairoRasterSourcePattern;
+extern PyTypeObject PycairoRasterSourcePattern_Type;
+
 extern PyTypeObject PycairoRectangleInt_Type;
 PyObject *PycairoRectangleInt_FromRectangleInt (
     const cairo_rectangle_int_t *rectangle_int);
@@ -82,11 +95,38 @@ PyObject *PycairoRectangleInt_FromRectangleInt (
 extern PyTypeObject PycairoRegion_Type;
 PyObject *PycairoRegion_FromRegion (cairo_region_t *region);
 
+extern PyTypeObject PycairoDevice_Type;
+PyObject *PycairoDevice_FromDevice (cairo_device_t *device);
+extern PyTypeObject PycairoScriptDevice_Type;
+
 extern PyTypeObject PycairoScaledFont_Type;
 PyObject *PycairoScaledFont_FromScaledFont (cairo_scaled_font_t *scaled_font);
 
 extern PyTypeObject PycairoSurface_Type;
 extern PyTypeObject PycairoImageSurface_Type;
+extern PyTypeObject PycairoMappedImageSurface_Type;
+
+extern PyTypeObject PycairoGlyph_Type;
+typedef PyTupleObject PycairGlyph;
+
+extern PyTypeObject PycairoRectangle_Type;
+typedef PyTupleObject PycairRectangle;
+
+extern PyTypeObject PycairoTextCluster_Type;
+typedef PyTupleObject PycairTextCluster;
+
+extern PyTypeObject PycairoTextExtents_Type;
+typedef PyTupleObject PycairoTextExtents;
+
+typedef struct {
+    PyObject_HEAD
+    cairo_device_t *device;
+} PycairoDevice;
+
+#if CAIRO_HAS_SCRIPT_SURFACE
+typedef PycairoDevice PycairoScriptDevice;
+extern PyTypeObject PycairoScriptSurface_Type;
+#endif
 
 #if CAIRO_HAS_PDF_SURFACE
 extern PyTypeObject PycairoPDFSurface_Type;
@@ -122,6 +162,11 @@ extern PyObject *xpybVISUALTYPE_type;
 extern PyTypeObject PycairoXlibSurface_Type;
 #endif
 
+#if CAIRO_HAS_TEE_SURFACE
+typedef PycairoSurface PycairoTeeSurface;
+extern PyTypeObject PycairoTeeSurface_Type;
+#endif
+
 PyObject *PycairoSurface_FromSurface (cairo_surface_t *surface,
                                       PyObject *base);
 
@@ -172,6 +217,15 @@ int Pycairo_Check_Status (cairo_status_t status);
     }								\
   } while (0)
 
+#define RETURN_NULL_IF_CAIRO_DEVICE_ERROR(device)	        \
+  do {								\
+    cairo_status_t status = cairo_device_status (device);	\
+    if (status != CAIRO_STATUS_SUCCESS) {			\
+      Pycairo_Check_Status (status);				\
+      return NULL;						\
+    }								\
+  } while (0)
+
 #define RETURN_NULL_IF_CAIRO_SCALED_FONT_ERROR(sc_font)             \
   do {								    \
     cairo_status_t status = cairo_scaled_font_status (sc_font);	    \
@@ -228,6 +282,8 @@ DECL_ENUM(PathDataType)
 DECL_ENUM(RegionOverlap)
 DECL_ENUM(Status)
 DECL_ENUM(SubpixelOrder)
+DECL_ENUM(TextClusterFlags)
+DECL_ENUM(SurfaceObserverMode)
 #ifdef CAIRO_HAS_SVG_SURFACE
 DECL_ENUM(SVGVersion)
 #endif
@@ -237,5 +293,8 @@ DECL_ENUM(PDFVersion)
 #ifdef CAIRO_HAS_PS_SURFACE
 DECL_ENUM(PSLevel)
 #endif
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+DECL_ENUM(ScriptMode)
+#endif
 
 #endif /* _PYCAIRO_PRIVATE_H_ */
